/**********************************************************************
 * tcp_socket_pipe.c                                     September 1999
 * Horms                                             horms@valinux.com.
 * VA Linux System                                 http://valinux.com./
 *
 * Functions to pipe data between two open file descriptors (sockets)
 *
 * Released under the terms of the GNU GPL
 *
 **********************************************************************/

#include "tcp_socket_pipe.h"

/**********************************************************************
 * tcp_socket_pipe
 * pipe data between two file discriptors until there is an error,
 * timeout or one or both the file descriptors are closed.
 * pre: fd_a_in:  one of the file descriptors to read from
 *      fd_a_out: one of the file descriptors to write to
 *      fd_b_in:  one of the file descriptors to read from
 *      fd_b_out: one of the file descriptors to write to
 *      buffer:   allocated buffer to read data into
 *      buffer_length: size of buffer in bytes
 * post: data is read from fd_a_in and written to fd_b_out and
 *       data is read from fd_b_in and written to fd_a_out.
 * return: -1 on error
 *         1 on idle timeout
 *         0 otherwise (one of fd_a_in or fd_b_in closes gracefully)
 **********************************************************************/

int tcp_socket_pipe(
  int fd_a_in, 
  int fd_a_out, 
  int fd_b_in, 
  int fd_b_out, 
  unsigned char *buffer, 
  int buffer_length
){
  fd_set read_template;
  fd_set except_template;
  struct timeval timeout;
  int status;
  
  extern int idle_timeout;

  for(;;){
    FD_ZERO(&read_template);
    FD_SET(fd_a_in, &read_template);
    FD_SET(fd_b_in, &read_template);

    FD_ZERO(&except_template);
    FD_SET(fd_a_in, &except_template);
    FD_SET(fd_b_in, &except_template);

    timeout.tv_sec=idle_timeout;
    timeout.tv_usec=0;

    status=select(
      FD_SETSIZE,
      &read_template,
      NULL,
      &except_template,
      idle_timeout?&timeout:NULL
    );
    if(status<0){
      if(errno!=EINTR){
        syslog(LOG_WARNING, "tcp_socket_pipe: select: %s", strerror(errno));
        return(-1);
      }
      return(0);
    }
    else if(
      FD_ISSET(fd_a_in, &except_template) ||
      FD_ISSET(fd_b_in, &except_template)
    ){
      syslog(LOG_WARNING, "tcp_socket_pipe: error on file descriptor");
      return(-1);
    }
    else if(status==0){
      syslog(LOG_WARNING, "tcp_socket_pipe: idle timeout");
      return(1);
    }
    else if(FD_ISSET(fd_a_in, &read_template)){
      if(tcp_socket_pipe_read_write(fd_a_in,fd_b_out,buffer,buffer_length)<0){
        return(-1);
      }
    }
    else if(FD_ISSET(fd_b_in, &read_template)){
      if(tcp_socket_pipe_read_write(fd_b_in,fd_a_out,buffer,buffer_length)<0){
        return(-1);
      }
    }
  }
}


/**********************************************************************
 * tcp_socket_pipe_read_write
 * Read data from one file descriptor (socket) and write it to another
 * pre: in_fd: file descriptor to read from
 *      out_fd: file descriptor to write to
 *      buffer: allocated buffer to store read data in
 *      buffer_length: size of the buffer
 * post: at most buffer_length bytes are read from in_fd and written 
 *       to out_fd. 
 * return: 0 on sucess
 *         -1 on error
 **********************************************************************/

int tcp_socket_pipe_read_write(
  int in_fd, 
  int out_fd, 
  unsigned char *buffer, 
  int buffer_length
){
  int bytes;

  bytes=read(in_fd, buffer, buffer_length);
  if(bytes<=0){
    return(-1);
  }
  write(out_fd, buffer, bytes);

  return(0);
}

